
var wescoPropertyTree= function(){

    this.get = function(path){
	return Property.getProperty(path);
    };

    this.store = function(){
	Property.store();
    };

    this.get_create = function(path,default_value,store){
	/*
	  This function retrieves a variable from the property
	  tree. If the variable does not exist, a new variable is
	  created with value default_value.

	  If store is true the property tree is stored

	  returns the read variable or undefined
	*/

	var myvar=this.get(path);

	if ((myvar === null||myvar === undefined)) {
	    myvar=default_value;
	    Property.setProperty(path,myvar);
    	    Property.setFlag(path, 'ARCHIVE', true); //set ARCHIVE flag to active
	    if (store){
		this.store();
	    }
	}

	return myvar;
    };


    this.save = function(path,var_value,store){
	/*
	  This function stores a value to the property tree.

	  path is the variable's  relative path.

	  If store is true, the property tree is stored
	*/

	var myvar=this.get(path);

	if (myvar === null || myvar === undefined) {
	    Property.setProperty(path,var_value);
	    Property.setFlag(path, 'ARCHIVE', true); //set flag to active
	    if (store){
		this.store();
	    }
	}else{
	    Property.setProperty(path,var_value);
	    if (store){
		this.store();
	    }
	}
    };

    this.setArchiveFlag = function(path){
	Property.setFlag(path, 'ARCHIVE', true); //set ARCHIVE flag to active
	this.store();
    };


    this.initialize = function(){
	/*
	  This function retrieves all the required variables from the
	  property tree.  Non-existent variables are created and assigned
	  a default value.
	*/


	//Notifications
	var Devices=[];

	this.get_create("Notifications/PushTargets",JSON.stringify(Devices),false);
	this.get_create("Notifications/PushTargetOptions",JSON.stringify(Devices),false);
	this.get_create("Notifications/EmailAddress",DEFAULT_STRING,false);


	// Controller
	//this.get_create("Controller/ClusterID",16,true);
	//this.save("Controller/ClusterID",19,false); // has to be created dynamically
	this.get_create("Controller/currentScene",DEFAULT_INT,false);


	// Exhaust flap
	this.get_create("ExhaustFlap",DEFAULT_STRING,false);
	//this.save("ExhaustFlap",DEFAULT_STRING,true);


	//Airboxes FilterReplacementInterval in [day]
	this.get_create("Airboxes/FilterReplacementInterval",FilterReplacementInterval,false); //365
	//this.save("Airboxes/FilterReplacementInterval",3,true); //365

	var dateReplacement=new Date();
	this.get_create("Airboxes/LastFilterReplacementDate",dateReplacement.toISOString(),false);

	//Prevent that app be uninstalled
	this.save("system/preventUninstall",false,false);



	// FMS variables
	this.get_create("Controller/BigModeState_current","Presence",false); //Presence,Absence,Alarm
	this.get_create("Controller/BigModeState_previous","Presence",false);
	this.get_create("Controller/PresenceState_current","Schedule",false); //Schedule,Manual,Cooking
	this.get_create("Controller/PresneceState_previous","Schedule",false);



	// Cluster (Ventilation UserGroup)
	var clusterID=this.get("Controller/ClusterID"); // has to be created dynamically
	// create cluster
	var clusterName="wescoBalanceSystem";

	if (clusterID===null || clusterID===undefined){
	    createCluster(clusterName);
	}else{
	    var name=Cluster.clusterGetName(clusterID);
	    if(name!=clusterName){
		createCluster(clusterName);
	    }
	}


	function createCluster(clusterName){
	    var colorGroup = 10; // ventilation
	    var ID = Cluster.createCluster(colorGroup,clusterName);
	    //Cluster.clusterAddDevice(ID,device-dSUID);
	    debug("DEBUG","ClusteName:"+name+";created ID:"+ID);
	    var lock = true;
	    Cluster.clusterConfigurationLock(ID,lock);
	    this.save("Controller/ClusterID",ID,true); // has to be created dynamically
	}


	this.store();
	debug("DEBUG","Loaded property tree");
    };


    this.deleteVentilationSchedule = function (){
	try{
	    var prop=Property.getNode("Controller/VentilationSchedule/");
	    var array = prop.getChildren();

	    for (var i in array){ // iterate over the airboxes nodes
		var Name=array[i].getName();
		var Path=array[i].getPath();

		var subprop=Property.getNode(Path);
		var subarray=subprop.getChildren();
		// iterate over all airbox properties and delete them
		for (var j in subarray){
		    var item=subarray[j].getName();
		    subprop.removeChild(item);
		}
		// once all the variables were removed, remove the
		// airbox node.
		prop.removeChild(Name);
	    }
	}
	catch(err){
	    //do nothing. For instance if the directory Devices does not yet exist
	}

    }

    this.configureVentilationSchedule = function (schedule){
	function is_aValidDay(d)
	{
	    var days=[0,1,2,3,4,5,6,7,8]; //[Sunday...Saturday,weekday,weekends]
	    return days.indexOf(parseInt(d)) >= 0;
	}

	try{
	    var dict=JSON.parse(schedule); // hours for a day
	    this.deleteVentilationSchedule(); // delete the previous Schedule

	    for (var i in dict){
		if(is_aValidDay(i)){
		    var daylist=JSON.stringify(dict[i]);
		    this.save("Controller/VentilationSchedule/"+i,daylist,false);

		}
	    }
	    this.store();
	}
	catch(err){
	    debug("DEBUG","exception;"+err+";");
	}
    };
};

var WescoVars=new wescoPropertyTree();  // WESCO property tree
